@@ -3,9 +3,8 @@ package MarpaX::Grammar::GraphViz2;
 use strict;
 use utf8;
 use warnings;
-use warnings  qw(FATAL utf8);    # Fatalize encoding glitches.
-use open      qw(:std :utf8);    # Undeclared streams in UTF-8.
-use charnames qw(:full :short);  # Unneeded in v5.16.
+use warnings  qw(FATAL utf8); # Fatalize encoding glitches.
+use open      qw(:std :utf8); # Undeclared streams in UTF-8.
 
 use File::Basename; # For basename().
 use File::Which; # For which().
@@ -16,15 +15,25 @@ use List::AllUtils qw/first_index indexes/;
 
 use Log::Handler;
 
+use Types::Standard qw/Any Bool HashRef Int Str/;
+
 use MarpaX::Grammar::Parser;
 
 use Moo;
 
+has bnf_name =>
+(
+	default  => sub{return 'BNF'},
+	is       => 'rw',
+	isa      => Str,
+	required => 0,
+);
+
 has default_count =>
 (
 	default  => sub{return 0},
 	is       => 'rw',
-	#isa     => 'Int',
+	isa      => Int,
 	required => 0,
 );
 
@@ -32,7 +41,7 @@ has discard_count =>
 (
 	default  => sub{return 0},
 	is       => 'rw',
-	#isa     => 'Int',
+	isa      => Int,
 	required => 0,
 );
 
@@ -40,7 +49,7 @@ has driver =>
 (
 	default  => sub{return ''},
 	is       => 'rw',
-	#isa     => 'Str',
+	isa      => Str,
 	required => 0,
 );
 
@@ -48,7 +57,7 @@ has event_count =>
 (
 	default  => sub{return 0},
 	is       => 'rw',
-	#isa     => 'Int',
+	isa      => Int,
 	required => 0,
 );
 
@@ -56,7 +65,15 @@ has format =>
 (
 	default  => sub{return ''},
 	is       => 'rw',
-	#isa     => 'Str',
+	isa      => Str,
+	required => 0,
+);
+
+has g1_rules_seen =>
+(
+	default  => sub{return {} },
+	is       => 'rw',
+	isa      => HashRef,
 	required => 0,
 );
 
@@ -64,7 +81,7 @@ has graph =>
 (
 	default  => sub{return ''},
 	is       => 'rw',
-	#isa     => 'GraphViz',
+	isa      => Any, # Actually an Object.
 	required => 0,
 );
 
@@ -72,7 +89,7 @@ has legend =>
 (
 	default  => sub{return 0},
 	is       => 'rw',
-	#isa     => 'Int',
+	isa      => Int,
 	required => 0,
 );
 
@@ -80,15 +97,7 @@ has lexeme_count =>
 (
 	default  => sub{return 0},
 	is       => 'rw',
-	#isa     => 'Int',
-	required => 0,
-);
-
-has lexemes =>
-(
-	default  => sub{return {} },
-	is       => 'rw',
-	#isa     => 'HashRef',
+	isa      => Int,
 	required => 0,
 );
 
@@ -96,7 +105,7 @@ has logger =>
 (
 	default  => sub{return undef},
 	is       => 'rw',
-#	isa      => 'Str',
+	isa      => Any,
 	required => 0,
 );
 
@@ -104,7 +113,7 @@ has marpa_bnf_file =>
 (
 	default  => sub{return ''},
 	is       => 'rw',
-	#isa     => 'Str',
+	isa      => Str,
 	required => 1,
 );
 
@@ -112,7 +121,7 @@ has maxlevel =>
 (
 	default  => sub{return 'notice'},
 	is       => 'rw',
-#	isa      => 'Str',
+	isa      => Str,
 	required => 0,
 );
 
@@ -120,7 +129,7 @@ has minlevel =>
 (
 	default  => sub{return 'error'},
 	is       => 'rw',
-#	isa      => 'Str',
+	isa      => Str,
 	required => 0,
 );
 
@@ -128,7 +137,7 @@ has nodes_seen =>
 (
 	default  => sub{return {} },
 	is       => 'rw',
-	#isa     => 'HashRef',
+	isa     => HashRef,
 	required => 0,
 );
 
@@ -136,7 +145,7 @@ has output_file =>
 (
 	default  => sub{return ''},
 	is       => 'rw',
-	#isa     => 'Str',
+	isa      => Str,
 	required => 0,
 );
 
@@ -144,23 +153,15 @@ has parser =>
 (
 	default  => sub{return ''},
 	is       => 'rw',
-	#isa     => 'MarpaX::Grammar::Parser',
+	isa      => Any, # Actually an Object.
 	required => 0,
 );
 
-has root_node =>
+has root_name =>
 (
-	default  => sub{return ''},
+	default  => sub{return 'DSL'},
 	is       => 'rw',
-	#isa     => 'Tree::DAG_Node',
-	required => 0,
-);
-
-has separators =>
-(
-	default  => sub{return {} },
-	is       => 'rw',
-	#isa     => 'HashRef',
+	isa      => Str,
 	required => 0,
 );
 
@@ -168,84 +169,19 @@ has user_bnf_file =>
 (
 	default  => sub{return ''},
 	is       => 'rw',
-	#isa     => 'Str',
+	isa      => Str,
 	required => 1,
 );
 
-our $VERSION = '1.05';
+has verbose =>
+(
+	default  => sub{return 0},
+	is       => 'rw',
+	isa      => Bool,
+	required => 1,
+);
 
-# ------------------------------------------------
-
-sub add_legend
-{
-	my($self) = @_;
-
-	$self -> graph -> push_subgraph
-	(
-		# No options...
-		# Legend: top. Border: no. Label: no.
-		#
-		# label => 'cluster_legend',
-		# Legend: top. Border: no. Label: no.
-		#
-		# name  => 'cluster_legend',
-		# Legend: top. Border: yes. Label: *.bnf.
-		#
-		#graph => {label => 'cluster_legend'},
-		# Legend: top. Border: no. Label: no. Not using subgraph => {...}.
-		#graph => {label => 'cluster_Legend'},
-		# Legend: bottom. Border: no. Label: no. Using subgraph => {...}.
-		# Legend: top. Border: no. Label: no. Not using subgraph => {...}.
-		subgraph => {rank => 'max'},
-		# Legend: top. Border: no. Label: no. Using graph => {...}.
-		# Legend: bottom. Border: no. Label: no. Not using graph => {...}.
-	);
-
-	$self -> graph -> add_node
-	(
-		label =>
-q|
-<<table bgcolor = 'white'>
-<tr>
-	<td bgcolor = 'lightgreen'>The green node is the start node</td>
-</tr>
-<tr>
-	<td bgcolor = 'lightblue'>Lightblue nodes are for lexeme attributes</td>
-</tr>
-<tr>
-	<td bgcolor = 'orchid'>Orchid nodes are for lexemes</td>
-</tr>
-<tr>
-	<td bgcolor = 'goldenrod'>Golden nodes are for actions</td>
-</tr>
-<tr>
-	<td bgcolor = 'firebrick1'>Red nodes are for events</td>
-</tr>
-</table>>
-|,
-		name  => 'Legend',
-		shape => 'plaintext',
-	);
-
-	$self -> graph -> pop_subgraph;
-
-} # End of add_legend.
-
-# ------------------------------------------------
-
-sub add_node
-{
-	my($self, %attributes) = @_;
-	my($name) = delete $attributes{name};
- 	my($seen) = $self -> nodes_seen;
-
-	$self -> graph -> add_node(name => $name, %attributes) if (! $$seen{$name});
-
-	$$seen{$name} = 1;
-
-	$self -> nodes_seen($seen);
-
-} # End of add_node.
+our $VERSION = '2.00';
 
 # ------------------------------------------------
 
@@ -283,7 +219,6 @@ sub BUILD
 		);
 
 	$self -> graph($graph);
-
 	$self -> parser
 	(
 		MarpaX::Grammar::Parser -> new
@@ -298,56 +233,82 @@ sub BUILD
 
 # ------------------------------------------------
 
-sub clean_name
-{
-	my($self, $name, $skip_symbols) = @_;
-
-	# If $skip_symbols is defined (as in calls from rectify_name() ),
-	# then do not do this check.
-
-	return $name if ( (! defined $skip_symbols) && ($name =~ /(?::=|=>|=)/) );
-
-	$name =~ s/\\/\\\\/g;             # Escape \.
-	$name =~ s/</\\</g;               # Escape <.
-	$name =~ s/>/\\>/g;               # Escape >.
-	$name =~ s/:/\x{a789}/g;          # Escape :.
-	$name =~ s/\"/\x{a78c}\x{a78c}/g; # Convert " into Unicode ' x 2.
-
-	return $name;
-
-} # End of clean_name.
-
-# ------------------------------------------------
-
-sub clean_tree
+sub add_legend
 {
 	my($self) = @_;
 
-	my($attributes);
-	my($name);
+	$self -> graph -> push_subgraph
+	(
+		# No options...
+		# Legend: top. Border: no. Label: no.
+		#
+		# label => 'cluster_legend',
+		# Legend: top. Border: no. Label: no.
+		#
+		# name  => 'cluster_legend',
+		# Legend: top. Border: yes. Label: *.bnf.
+		#
+		#graph => {label => 'cluster_legend'},
+		# Legend: top. Border: no. Label: no. Not using subgraph => {...}.
+		#graph => {label => 'cluster_Legend'},
+		# Legend: bottom. Border: no. Label: no. Using subgraph => {...}.
+		# Legend: top. Border: no. Label: no. Not using subgraph => {...}.
+		subgraph => {rank => 'max'},
+		# Legend: top. Border: no. Label: no. Using graph => {...}.
+		# Legend: bottom. Border: no. Label: no. Not using graph => {...}.
+	);
 
-	$self -> parser -> cooked_tree -> walk_down
-	({
-		callback => sub
-		{
-			# Elsewhere, the code uses these attributes (so we init them if necessary):
-			# o name.
-			# o quantifier.
-			# o real_name.
+	$self -> graph -> add_node
+	(
+		label =>
+q|
+<<table bgcolor = 'white'>
+<tr>
+	<td bgcolor = 'pink'>Pink nodes are ':default' rules</td>
+</tr>
+<tr>
+	<td bgcolor = 'magenta'>Magenta nodes are ':discard' rules</td>
+</tr>
+<tr>
+	<td bgcolor = 'orchid'>The orchid node is the 'discard default' rule</td>
+</tr>
+<tr>
+	<td bgcolor = 'goldenrod'>The golden node is the 'lexeme default' rule</td>
+</tr>
+<tr>
+	<td bgcolor = 'lightblue'>The LightBlue node is the ':start' rule</td>
+</tr>
+<tr>
+	<td bgcolor = 'CornflowerBlue'>CornflowerBlue nodes are G1 rules</td>
+</tr>
+<tr>
+	<td bgcolor = 'lightgreen'>LightGreen nodes are ':lexeme' rules</td>
+</tr>
+</table>>
+|,
+		name  => 'Legend',
+		shape => 'plaintext',
+	);
 
-			my($node, $options) = @_;
-			$name               = $node -> name;
-			$attributes         = {name => '', quantifier => '', real_name => '', %{$node -> attributes} };
+	$self -> graph -> pop_subgraph;
 
-			$node -> attributes($attributes);
-			$node -> name($self -> clean_name($name) );
+} # End of add_legend.
 
-			return 1; # Keep walking.
-		},
-		_depth => 0,
-	});
+# ------------------------------------------------
 
-} # End of clean_tree.
+sub add_node
+{
+	my($self, %attributes) = @_;
+	my($name) = delete $attributes{name};
+ 	my($seen) = $self -> nodes_seen;
+
+	$self -> graph -> add_node(name => $name, %attributes) if (! $$seen{$name});
+
+	$$seen{$name} = 1;
+
+	$self -> nodes_seen($seen);
+
+} # End of add_node.
 
 # ------------------------------------------------
 
@@ -375,138 +336,163 @@ sub log
 sub _process_adverbs
 {
 	my($self, $daughters) = @_;
-	my($end)        = $#$daughters;
-	my($separators) = $self -> separators;
 
-	# Chop adverbs off the end of the list.
+	my($attr);
+	my(@adverbs);
+	my(@grand_kids);
+	my($name);
+	my($token_1, $token_2);
 
-	my($adverb, @adverbs);
-	my(@token);
-
-	while ($end > 0)
+	for my $j (2 .. $#$daughters)
 	{
-		if ($$daughters[$end - 1] -> name eq '=>')
-		{
-			$adverb = $$daughters[$end - 2] -> name;
+		$name       = $$daughters[$j] -> name;
+		$attr       = $$daughters[$j] -> attributes;
+		$token_1    = $$attr{token};
+		@grand_kids = $$daughters[$j] -> daughters;
 
-			# rectify_name() returns a ($name => $label) pair.
+		last if ($#grand_kids < 0); # TODO. Alternative.
 
-			@token = $self -> rectify_name($$daughters[$end]);
+		$attr    = ($$daughters[$j] -> daughters)[0] -> attributes;
+		$token_2 = $$attr{token};
 
-			pop @$daughters for 1 .. 3;
-
-			push @adverbs,
-			{
-				adverb => $adverb,
-				name   => $token[0],
-			};
-
-			$end                    = $#$daughters;
-			$$separators{$token[0]} = 1 if ($adverb eq 'separator');
-		}
-		else
-		{
-			$end = 0;
-		}
+		push @adverbs, "$token_1 =\\> $token_2";
 	}
 
-	# Construct the label as an array of hashrefs.
-
 	if ($#adverbs >= 0)
 	{
-		@adverbs            = map{"$$_{adverb} =\\> $$_{name}"} @adverbs;
 		$adverbs[0]         = "\{$adverbs[0]";
 		$adverbs[$#adverbs] .= '}';
 		@adverbs            = map{ {text => $_} } @adverbs;
 	}
 
-	return ([@$daughters], [@adverbs]);
+	return [@adverbs];
 
 } # End of _process_adverbs.
 
 # ------------------------------------------------
-# This handles prioritized rules and quantized rules.
 
-sub _process_complex_adverbs
+sub _process_bare_lexeme
 {
-	my($self, $index, $a_node) = @_;
+	my($self, $daughters) = @_;
 
-	# Sample inputs:
-	# o 1 token, no adverbs:
-	#   |--- number
-	#   |   |--- ~
-	#   |   |--- int
-	# o 2 tokens, no adverbs:
-	#   |--- json
-	#   |   |--- ::=
-	#   |   |--- object
-	#   |   |--- |
-	#   |   |--- array
-	# o 5 tokens, various adverbs:
-	#   |--- object
-	#   |   |--- ::=
-	#   |   |--- '{'
-	#   |   |--- '}'
-	#   |   |--- action
-	#   |   |--- =>
-	#   |   |--- do_empty_object
-	#   |   |--- |
-	#   |   |--- '{'
-	#   |   |--- members
-	#   |   |--- '}'
-	#   |   |--- action
-	#   |   |--- =>
-	#   |   |--- do_object
+} # End of _process_bare_lexeme.
 
-	my($daughters) = [$a_node -> daughters];
-	my($finished)  = $#$daughters >= 0 ? 0 : 1;
+# ------------------------------------------------
 
-	my($adverbs, @adverb_stack);
-	my(@daughter_stack);
-	my(@token_stack);
-
-	while (! $finished)
+sub _process_bare_rule
+{
+	my($self, $daughters) = @_;
+	my($attr)       = ($$daughters[0] -> daughters)[0] -> attributes;
+	my($token_1)    = $$attr{token};
+	my($attributes) =
 	{
-		# Chew adverbs, if any, off the end of the list of daughters.
+		fillcolor => 'LightSkyBlue',
+		label     => $token_1,
+	};
 
-		($daughters, $adverbs) = $self -> _process_adverbs($daughters);
+	$self -> add_node(name => $token_1, %$attributes);
 
-		# Stack the adverbs owned by the token(s) at the end of the daughters.
+	# Loop over the alternative branches within this rule.
+	# Note: There might by N rule tokens before 'alternative'.
 
-		unshift @adverb_stack, $adverbs;
+	my($rules_seen) = $self -> g1_rules_seen;
 
-		# Chew the tokens owning the adverbs off the end of the list of daughters.
-		# This backward processing stops with a '|' or $daughter[0].
+	my(@hidden_tokens, @hidden_kids);
+	my(@kids);
+	my(@label);
+	my($name);
+	my($rule_name);
+	my($token_2, $token_3);
 
-		@token_stack = ();
+	for (my $index = 2; $index <= $#$daughters; $index++)
+	{
+		$name = $$daughters[$index] -> name;
 
-		my($i) = $#$daughters;
-
-		while ( ($i > 0) && ($$daughters[$i] -> name ne '|') )
+		if ($name eq 'alternative')
 		{
-			unshift @token_stack, pop @$daughters;
+			$rule_name          = join(' ', map{s/:/\x{a789}/; $_} @label);
+			$$attributes{label} = $rule_name;
 
-			$i--;
+			$self -> add_node(name => $rule_name, %$attributes);
+			$self -> graph -> add_edge(from => $token_1, to => $rule_name);
+
+			for (@label)
+			{
+				next if ($_ eq $rule_name);
+
+				$$attributes{label} = $_;
+
+				$self -> add_node(name => $_, %$attributes);
+				$self -> graph -> add_edge(from => $rule_name, to => $_);
+			}
+
+			@label = ();
+
+			next;
+		}
+		elsif ($name eq 'parenthesized_rhs_primary_list')
+		{
+			@hidden_kids   = $$daughters[$index] -> daughters;
+			@hidden_tokens = ('(');
+
+			for (my $i = 0; $i <= $#hidden_kids; $i++)
+			{
+				$attr = $hidden_kids[$i] -> attributes;
+
+				push @hidden_tokens, $$attr{token};
+			}
+
+			push @hidden_tokens, ')';
+			push @label, join(' ', @hidden_tokens);
+
+			$index++;
+
+			next;
+		}
+		elsif ($name ne 'rhs')
+		{
+			next;
 		}
 
-		unshift @daughter_stack, [@token_stack];
+		$attr = $$daughters[$index] -> attributes;
+		@kids = $$daughters[$index] -> daughters;
 
-		# Discard the '|' separating alternatives in the BND.
+		next if ($#kids < 0);
 
-		pop @$daughters if ( ($i >= 0) && ($$daughters[$i] -> name eq '|') );
+		$attr    = $kids[0] -> attributes;
+		$token_2 = $$attr{token};
 
-		$finished = 1 if ($#$daughters == 0);
+		push @label, $token_2;
 	}
 
-	return ([@daughter_stack], [@adverb_stack]);
+	if ($#label >= 0)
+	{
+		$rule_name          = join(' ', map{s/:/\x{a789}/g; $_} @label);
+		$$attributes{label} = $rule_name;
 
-} # End of _process_complex_adverbs.
+		$self -> add_node(name => $rule_name, %$attributes);
+		$self -> graph -> add_edge(from => $token_1, to => $rule_name);
+
+		for (@label)
+		{
+			next if ($_ eq $rule_name);
+
+			$$attributes{label} = $_;
+
+			$self -> add_node(name => $_, %$attributes);
+			$self -> graph -> add_edge(from => $rule_name, to => $_);
+		}
+	}
+
+	$self -> g1_rules_seen($rules_seen);
+
+} # End of _process_bare_rule.
 
 # ------------------------------------------------
 
 sub _process_default_rule
 {
-	my($self, $index, $a_node) = @_;
+	my($self, $daughters) = @_;
 
 	$self -> default_count($self -> default_count + 1);
 
@@ -514,21 +500,21 @@ sub _process_default_rule
 	my($default_name)  = "\x{a789}default";
 	my($attributes)    =
 	{
-		fillcolor => 'lightblue',
-		label     => $default_name,
+		fillcolor => 'pink',
+		label     => [{text => " $default_name"}], # Warning: Don't delete the ' ' before the \x.
 	};
 
 	if ($default_count == 1)
 	{
 		$self -> add_node(name => $default_name, %$attributes);
-		$self -> graph -> add_edge(from => $self -> root_node -> name, to => $default_name);
+		$self -> graph -> add_edge(from => $self -> root_name, to => $default_name);
 	}
 
-	my($daughters, $adverbs) = $self -> _process_adverbs([$a_node -> daughters]);
+	my($adverbs) = $self -> _process_adverbs($daughters);
 
 	if ($#$adverbs >= 0)
 	{
-		$$attributes{fillcolor} = 'goldenrod';
+		$$attributes{fillcolor} = 'pink';
 		$$attributes{label}     = $adverbs;
 		my($adverb_name)        = "${default_name}_$default_count";
 
@@ -540,9 +526,39 @@ sub _process_default_rule
 
 # ------------------------------------------------
 
+sub _process_discard_default_rule
+{
+	my($self, $daughters) = @_;
+	my($discard_name) = "discard default";
+	my($attributes)   =
+	{
+		fillcolor => 'orchid',
+		label     => $discard_name,
+	};
+
+	$self -> add_node(name => $discard_name, %$attributes);
+	$self -> graph -> add_edge(from => $self -> root_name, to => $discard_name);
+
+	my($adverbs) = $self -> _process_adverbs($daughters);
+
+	if ($#$adverbs >= 0)
+	{
+		$$attributes{fillcolor} = 'orchid';
+		$$attributes{label}     = $adverbs;
+		my($adverb_name)        = "${discard_name}_1";
+
+		$self -> add_node(name => $adverb_name, %$attributes);
+		$self -> graph -> add_edge(from => $discard_name, to => $adverb_name);
+	}
+
+
+} # End of _process_discard_default_rule.
+
+# ------------------------------------------------
+
 sub _process_discard_rule
 {
-	my($self, $index, $a_node) = @_;
+	my($self, $daughters) = @_;
 
 	$self -> discard_count($self -> discard_count + 1);
 
@@ -550,92 +566,76 @@ sub _process_discard_rule
 	my($discard_name)  = "\x{a789}discard";
 	my($attributes)    =
 	{
-		fillcolor => 'lightblue',
-		label     => $discard_name,
+		fillcolor => 'magenta',
+		label     => [{text => " $discard_name"}], # Warning: Don't delete the ' ' before the \x.
 	};
 
 	if ($discard_count == 1)
 	{
 		$self -> add_node(name => $discard_name, %$attributes);
-		$self -> graph -> add_edge(from => $self -> root_node -> name, to => $discard_name);
+		$self -> graph -> add_edge(from => $self -> root_name, to => $discard_name);
 	}
 
-	# Ignore the first daughter, which is '=>'.
+	my($attr)  = ($$daughters[2] -> daughters)[0] -> attributes;
+	my($token) = $$attr{token};
 
-	my(@daughters) = $a_node -> daughters;
+	$$attributes{fillcolor} = 'magenta';
+	$$attributes{label}     = $token;
 
-	# rectify_name() returns a ($name => $label) pair.
-
-	my(@name)           = $self -> rectify_name($daughters[1]);
-	$$attributes{label} = $name[1];
-
-	$self -> add_node(name => $name[0], %$attributes);
-	$self -> graph -> add_edge(from => $discard_name, to => $name[0]);
+	$self -> add_node(name => $token, %$attributes);
+	$self -> graph -> add_edge(from => $discard_name, to => $token);
 
 } # End of _process_discard_rule.
 
 # ------------------------------------------------
 
-sub _process_event_rule
+sub _process_lexeme_adverbs
 {
-	my($self, $index, $a_node) = @_;
+	my($self, $daughters) = @_;
 
-	$self -> event_count($self -> event_count + 1);
+	my($attr);
+	my(@adverbs);
+	my(@grand_kids);
+	my($name);
+	my($token_1, $token_2);
 
-	my($event_count) = $self -> event_count;
-	my($event_name)  = 'event';
-	my($attributes)  =
+	for my $j (2 .. $#$daughters)
 	{
-		fillcolor => 'firebrick1',
-		label     => $event_name,
-	};
+		$name       = $$daughters[$j] -> name;
+		$attr       = $$daughters[$j] -> attributes;
+		$token_1    = $$attr{token};
+		$attr       = ($$daughters[$j] -> daughters)[0] -> attributes;
+		$token_2    = $$attr{token};
 
-	if ($event_count == 1)
-	{
-		$self -> add_node(name => $event_name, %$attributes);
-		$self -> graph -> add_edge(from => $self -> root_node -> name, to => $event_name);
+		push @adverbs, [$token_1, $token_2];
 	}
 
-	my($item_name) = "${event_name}_$event_count";
-	my(@daughters) = $a_node -> daughters;
+	return [@adverbs];
 
-	# rectify_name() returns a ($name => $label) pair.
-
-	my(@lhs)            = $self -> rectify_name($daughters[3]);
-	$$attributes{label} =
-	[
-		{text => '{' . $daughters[0] -> name},
-		{text => $daughters[2] -> name},
-		{text => "$lhs[0]}"},
-	];
-
-	$self -> add_node(name => $item_name, %$attributes);
-	$self -> graph -> add_edge(from => $event_name, to => $item_name);
-	$self -> graph -> add_edge(from => $item_name, to => $lhs[0]);
-
-} # End of _process_event_rule.
+} # End of _process_lexeme_adverbs.
 
 # ------------------------------------------------
 
 sub _process_lexeme_default_rule
 {
-	my($self, $index, $a_node) = @_;
+	my($self, $daughters) = @_;
 	my($lexeme_name) = 'lexeme default';
 	my($attributes)  =
 	{
-		fillcolor => 'lightblue',
+		fillcolor => 'goldenrod',
 		label     => $lexeme_name,
 	};
 
 	$self -> add_node(name => $lexeme_name, %$attributes);
-	$self -> graph -> add_edge(from => $self -> root_node -> name, to => $lexeme_name);
+	$self -> graph -> add_edge(from => $self -> root_name, to => $lexeme_name);
 
-	my($daughters, $adverbs) = $self -> _process_adverbs([$a_node -> daughters]);
+	my($adverbs) = $self -> _process_adverbs($daughters);
 
 	if ($#$adverbs >= 0)
 	{
-		$$attributes{label} = $adverbs;
-		my($adverb_name)    = "${lexeme_name}_1";
+		$$attributes{fillcolor} = 'goldenrod';
+		$$attributes{label}     = $adverbs;
+		my($adverb_name)        = "${lexeme_name}_1";
 
 		$self -> add_node(name => $adverb_name, %$attributes);
 		$self -> graph -> add_edge(from => $lexeme_name, to => $adverb_name);
@@ -647,160 +647,62 @@ sub _process_lexeme_default_rule
 
 sub _process_lexeme_rule
 {
-	my($self, $index, $a_node) = @_;
-	my($daughters, $adverbs)   = $self -> _process_adverbs([$a_node -> daughters]);
+	my($self, $daughters) = @_;
 
-	# rectify_name() returns a ($name => $label) pair.
+	$self -> lexeme_count($self -> lexeme_count + 1);
 
-	my(@name)          = $self -> rectify_name($$daughters[1]);
-	my($lexeme)        = $self -> lexemes;
-	$$lexeme{$name[0]} = $#$adverbs >= 0 ? $adverbs : '';
+	my($adverbs)      = $self -> _process_lexeme_adverbs($daughters);
+	my($lexeme_count) = $self -> lexeme_count;
+	my($lexeme_name)  = shift @$adverbs;
+	$lexeme_name      = $$lexeme_name[1]; # Discard 'bare_name'.
+	my($attributes)   =
+	{
+		fillcolor => 'DeepSkyBlue2',
+		label     => [{text => " $lexeme_name"}], # Warning: Don't delete the ' ' before the \x.
+	};
 
-	$self -> lexemes($lexeme);
+	if ($#$adverbs >= 0)
+	{
+		@$adverbs = map{"$$_[0] =\\> $$_[1]"} @$adverbs;
+
+		unshift @$adverbs, " \x{a789}lexeme =\\> $lexeme_name";
+
+		$$adverbs[0]          = "\{$$adverbs[0]";
+		$$adverbs[$#$adverbs] .= '}';
+		@$adverbs             = map{ {text => $_} } @$adverbs;
+		$$attributes{label}   = $adverbs;
+		my($adverb_name)      = "${lexeme_name}_$lexeme_count";
+
+		$self -> add_node(name => $adverb_name, %$attributes);
+		$self -> graph -> add_edge(from => $lexeme_name, to => $adverb_name);
+	}
 
 } # End of _process_lexeme_rule.
 
 # ------------------------------------------------
 
-sub _process_lexeme_token
-{
-	my($self, $index, $lexemes, $name, $label) = @_;
-
-	my($attributes);
-
-	if ($$lexemes{$name})
-	{
-		$attributes =
-		{
-			fillcolor => 'orchid',
-			label     => [{text => "\{\x{a789}lexeme"}, {text => "$label}"}],
-		};
-	}
-	else
-	{
-		$attributes =
-		{
-			fillcolor => 'white',
-			label     => $label,
-		};
-	}
-
-	return $attributes;
-
-} # End of _process_lexeme_token.
-
-# ------------------------------------------------
-
-sub _process_normal_rule
-{
-	my($self, $index, $a_node, $lexemes) = @_;
-	my($daughters, $adverbs) = $self -> _process_complex_adverbs($index, $a_node);
-
-	for my $i (0 .. $#$daughters)
-	{
-		if ($#{$$daughters[$i]} >= 0)
-		{
-			$self -> _process_normal_tokens($index, $a_node, $lexemes, $$daughters[$i], $$adverbs[$i]);
-		}
-	}
-
-} # End of _process_normal_rule.
-
-# ------------------------------------------------
-
-sub _process_normal_tokens
-{
-	my($self, $index, $a_node, $lexemes, $daughters, $adverbs) = @_;
-
-	# rectify_name() returns a ($name => $label) pair.
-
-	my(@name_map)   = map{$self -> rectify_name($_)} @$daughters;
-	my(@name)       = map{$name_map[$_]} indexes{$_ % 2 == 0} 0 .. $#name_map;
-	my(@label)      = map{$name_map[$_]} indexes{$_ % 2 != 0} 0 .. $#name_map;
-	my($rule_name)  = join(' ', @name);
-	my($rule_label) = join(' ', @label);
-	my($attributes) = $self -> _process_lexeme_token($index, $lexemes, $rule_name, $rule_label);
-	my(@parent)     = $self -> rectify_name($a_node);
-
-	$self -> add_node(name => $rule_name, %$attributes);
-	$self -> graph -> add_edge(from => $parent[0], to => $rule_name);
-
-	my($attr_name);
-	my($name);
-
-	for my $i (0 .. $#name)
-	{
-		$name = $name[$i];
-
-		$attributes = $self -> _process_lexeme_token($index, $lexemes, $name, $label[$i]);
-
-		# Don't re-add the node added just above.
-		# This happens in cases where there is just 1 daughter,
-		# which mean the join() above only had 1 name to 'join'.
-		# Nevertheless, after this 'if', we still add its attributes (outside the loop).
-
-		if ($name ne $rule_name)
-		{
-			$self -> add_node(name => $name, %$attributes);
-			$self -> graph -> add_edge(from => $rule_name, to => $name);
-		}
-
-		if ($$lexemes{$name})
-		{
-			$$attributes{fillcolor} = 'lightblue';
-			$$attributes{label}     = $$lexemes{$name};
-			$attr_name              = "${name}_$i";
-
-			$self -> add_node(name => $attr_name, %$attributes);
-			$self -> graph -> add_edge(from => $name, to => $attr_name);
-		}
-	}
-
-	if ($#$adverbs >= 0)
-	{
-		$$attributes{fillcolor} = 'goldenrod';
-		$$attributes{label}     = $adverbs;
-		$attr_name              = "${rule_name}_attributes";
-
-		$self -> add_node(name => $attr_name, %$attributes);
-		$self -> graph -> add_edge(from => $rule_name, to => $attr_name);
-	}
-
-} # End of _process_normal_tokens.
-
-# ------------------------------------------------
-
 sub _process_start_rule
 {
-	my($self, $index, $a_node) = @_;
-	my(@daughters) = $a_node -> daughters;
-	my(@name)      = $self -> rectify_name($daughters[1]);
-
-	$self -> root_node($daughters[1]);
-
+	my($self, $daughters) = @_;
+	my($name)       = 'start';
 	my($attributes) =
 	{
-		fillcolor => 'lightgreen',
-		label     => [{text => '{:start'}, {text => "$name[1]}"}],
+		fillcolor => 'Turquoise1',
+		label     => [{text => " \x{a789}start"}], # Warning: Don't delete the ' ' before the \x.
 	};
 
-	$self -> add_node(name => $name[0], %$attributes);
+	$self -> add_node(name => $name, %$attributes);
+	$self -> graph -> add_edge(from => $self -> root_name, to => $name);
+
+	my($attr)           = ($$daughters[2] -> daughters)[0] -> attributes;
+	my($starter)        = $$attr{token};
+	$$attributes{label} = [{text => $$attr{token} }];
+
+	$self -> add_node(name => $starter, %$attributes);
+	$self -> graph -> add_edge(from => $name, to => $starter);
 
 } # End of _process_start_rule.
 
-# ------------------------------------------------
-
-sub rectify_name
-{
-	my($self, $node) = @_;
-	my($attributes)  = $node -> attributes;
-	my($name)        = $self -> clean_name($$attributes{real_name}, 1);
-	my($label)       = $name . $$attributes{quantifier};
-
-	return ($name, $label);
-
-} # End of rectify_name.
-
 # ----------------------------------------------
 
 sub run
@@ -812,58 +714,72 @@ sub run
 
 	return $result if ($result == 1);
 
-	$self -> clean_tree;
+	# Look over the statements.
 
-	#$self -> log(debug => $_) for @{$self -> parser -> cooked_tree -> tree2string({no_attributes => 0})};
+	my(@statements) = $self -> parser -> cooked_tree -> daughters;
 
-	my(@rule)        = $self -> parser -> cooked_tree -> daughters;
-	my($start_index) = first_index{$_ -> name eq "\x{a789}start"} @rule;
+	my($attributes);
+	my(@daughters);
+	my($name);
+	my($offset);
+	my($token);
 
-	# Warning: This must be first because it sets $self -> root_node().
-
-	$self -> _process_start_rule($start_index + 1, $rule[$start_index]);
-
-	for my $index (indexes {$_ -> name eq "\x{a789}default"} @rule)
+	for my $i (0 .. $#statements)
 	{
-		$self -> _process_default_rule($index + 1, $rule[$index]);
-	}
+		if ($self -> verbose)
+		{
+			print "i: @{[$i + 1]}. name: ", $statements[$i] -> name, ". \n";
+		}
 
-	for my $index (indexes {$_ -> name eq "\x{a789}discard"} @rule)
-	{
-		$self -> _process_discard_rule($index + 1, $rule[$index]);
-	}
+		# Loop over the components of a single statement.
 
-	my($lexeme_default_index) = first_index{$_ -> name eq 'lexeme default'} @rule;
+		@daughters  = $statements[$i] -> daughters;
+		$name       = $daughters[0] -> name;
+		$attributes = $daughters[0] -> attributes;
+		$token      = $$attributes{token};
 
-	$self -> _process_lexeme_default_rule($lexeme_default_index + 1, $rule[$lexeme_default_index]) if ($lexeme_default_index >= 0);
+		next if ($name ne 'lhs');
 
-	for my $index (indexes {$_ -> name eq "\x{a789}lexeme"} @rule)
-	{
-		$self -> _process_lexeme_rule($index + 1, $rule[$index]);
-	}
+		if ($self -> verbose)
+		{
+			print "\t1st daughter. name: $name. token: $token. \n";
+		}
 
-	my(%seen) =
-	(
-		"\x{a789}default" => 1,
-		"\x{a789}discard" => 1,
-		'event'           => 1, # Not yet, but we want process_normal_rule() to skip events.
-		"\x{a789}lexeme"  => 1,
-		'lexeme default'  => 1,
-		"\x{a789}start"   => 1,
-	);
-
-	my($lexemes) = $self -> lexemes;
-
-	for my $index (0 .. $#rule)
-	{
-		next if ($seen{$rule[$index] -> name});
-
-		$self -> _process_normal_rule($index + 1, $rule[$index], $lexemes);
-	}
-
-	for my $index (indexes {$_ -> name eq 'event'} @rule)
-	{
-		$self -> _process_event_rule($index + 1, $rule[$index]);
+		if ($token eq 'bare_name')
+		{
+			if ($daughters[1] -> name eq 'op_declare_bnf')
+			{
+				$self -> _process_bare_rule(\@daughters);
+			}
+			else
+			{
+				$self -> _process_bare_lexeme(\@daughters);
+			}
+		}
+		elsif ($token eq ':default')
+		{
+			$self -> _process_default_rule(\@daughters);
+		}
+		elsif ( ($token eq 'discard default') && ($name eq 'lhs') )
+		{
+			$self -> _process_discard_default_rule(\@daughters);
+		}
+		elsif ($token eq ':discard')
+		{
+			$self -> _process_discard_rule(\@daughters);
+		}
+		elsif ($token eq 'lexeme default')
+		{
+			$self -> _process_lexeme_default_rule(\@daughters);
+		}
+		elsif ($token eq ':lexeme')
+		{
+			$self -> _process_lexeme_rule(\@daughters);
+		}
+		elsif ($token eq ':start')
+		{
+			$self -> _process_start_rule(\@daughters);
+		}
 	}
 
 	$self -> add_legend if ($self -> legend);
@@ -1233,9 +1149,9 @@ of the node, when the node is finally passed to L<GraphViz2>.
 
 Returns a 2-element list of ($name, $label).
 
-=head2 root_node()
+=head2 root_name()
 
-Returns an object of type L<Tree::DAG_Node>, representing the C<:start> token in the user's grammar.
+Returns a string which is the name of the root node of graph.
 
 =head2 run()
 
@@ -1243,13 +1159,6 @@ The method which does all the work.
 
 See L</Synopsis> and scripts/bnf2graph.pl for sample code.
 
-=head2 separators()
-
-Returns a hashref keyed by token name, of tokens used in the grammar construc
- C<< separator => $token >>.
-
-This hashref is currently not used.
-
 =head2 user_bnf_file([$bnf_file_name])
 
 Here, the [] indicate an optional parameter.
@@ -1274,12 +1183,6 @@ This is the image from c.ast.bnf.
 
 See the next point for how this file is created.
 
-=item o share/c.ast.log
-
-This is the log produced by running the code at log level C<debug>:
-
-	shell> scripts/bnf2graph.sh c.ast -max debug > share/c.ast.log
-
 =item o share/json.1.bnf
 
 It is part of L<MarpaX::Demo::JSONParser>, written as a gist by Peter Stuifzand.
@@ -1288,12 +1191,6 @@ See L<https://gist.github.com/pstuifzand/4447349>.
 
 See the next point for how this file is created.
 
-=item o share/json.1.log
-
-This is the log produced by running the code at log level C<debug>:
-
-	shell> scripts/bnf2graph.sh json.1 -max debug > share/json.1.log
-
 =item o html/json.1.svg
 
 This is the image from json.1.bnf.
@@ -1303,12 +1200,6 @@ This is the image from json.1.bnf.
 It also is part of L<MarpaX::Demo::JSONParser>, written by Jeffrey Kegler as a reply to the gist
 above from Peter.
 
-=item o share/json.2.log
-
-This is the log produced by running the code at log level C<debug>:
-
-	shell> scripts/bnf2graph.sh json.2 -max debug > share/json.2.log
-
 =item o html/json.2.svg
 
 This is the image from json.2.bnf.
@@ -1319,12 +1210,6 @@ See the previous point for how this file is created.
 
 It also is part of L<MarpaX::Demo::JSONParser>, and is written by Jeffrey Kegler.
 
-=item o share/json.3.log
-
-This is the log produced by running the code at log level C<debug>:
-
-	shell> scripts/bnf2graph.sh json.3 -max debug > share/json.3.log
-
 =item o html/json.3.svg
 
 This is the image from json.3.bnf.
@@ -1333,7 +1218,19 @@ This is the image from json.3.bnf.
 
 This is a copy of L<Marpa::R2>'s BNF, as of Marpa::R2 V 2.096000.
 
-See L</marpa_bnf_file([$bnf_file_name])> above.
+=item o html/metag.svg
+
+This is the image from metag.bnf.
+
+=item o share/numeric.expressions.bnf.
+
+This BNF was extracted from L<MarpaX::Demo::SampleScripts>'s examples/ambiguous.grammar.01.pl.
+
+=item o html/numeric.expressions.svg
+
+This is the image from numeric.expressions.bnf.
+
+See the next point for how this file is created.
 
 =item o share/stringparser.bnf.
 
@@ -1345,14 +1242,6 @@ This is the image from stringparser.bnf.
 
 See the next point for how this file is created.
 
-=item o share/stringparser.log
-
-This is the log produced by running the code at log level C<debug>:
-
-	shell> scripts/bnf2graph.sh stringparser -max debug > share/stringparser.log
-
-See L</user_bnf_file([$bnf_file_name])> above.
-
 =item o share/termcap.info.bnf
 
 It also is part of L<MarpaX::Database::Terminfo>, written by Jean-Damien Durand.
@@ -1363,12 +1252,6 @@ This is the image from termcap.info.bnf.
 
 See the next point for how this file is created.
 
-=item o share/termcap.info.log
-
-This is the log produced by running the code at log level C<debug>:
-
-	shell> scripts/bnf2graph.sh termcap.info -max debug > share/termcap.info.log
-
 =back
 
 =head2 Scripts
@@ -1512,7 +1395,7 @@ Australian copyright (c) 2013, Ron Savage.
 
 	All Programs of mine are 'OSI Certified Open Source Software';
 	you can redistribute them and/or modify them under the terms of
-	The Artistic License, a copy of which is available at:
+	The Perl License, a copy of which is available at:
 	http://www.opensource.org/licenses/index.html
 
 =cut
