@@ -3,16 +3,15 @@ package MarpaX::Grammar::GraphViz2::Filer;
 use strict;
 use utf8;
 use warnings;
-use warnings  qw(FATAL utf8);    # Fatalize encoding glitches.
-use open      qw(:std :utf8);    # Undeclared streams in UTF-8.
-use charnames qw(:full :short);  # Unneeded in v5.16.
+use warnings  qw(FATAL utf8); # Fatalize encoding glitches.
+use open      qw(:std :utf8); # Undeclared streams in UTF-8.
 
 use File::Basename; # For basename().
 use File::Slurp;    # For read_file().
 
 use Moo;
 
-our $VERSION = '1.05';
+our $VERSION = '2.00';
 
 # ------------------------------------------------
 
@@ -21,7 +20,7 @@ sub get_files
 	my($self, $dir_name, $type) = @_;
 
 	opendir(my $fh, $dir_name);
-	my(@file) = sort grep{! /metag.bnf/} grep{/$type$/} readdir $fh;
+	my(@file) = sort grep{/$type$/} readdir $fh;
 	closedir $fh;
 
 	my(%file);
@@ -71,8 +70,6 @@ Returns a hash (sic) of files from the given $dir_name, whose type (extension) m
 
 The hash is keyed by the file's basename. See L<File::Basename>.
 
-The file share/metag.bnf is omitted from the set returned.
-
 =head1 Version Numbers
 
 Version numbers < 1.00 represent development versions. From 1.00 up, they are production versions.
